/* simple dissociation of oxygen
   using SI units
   developed by Dudley J. Benton */

#define _CRT_SECURE_NO_DEPRECATE
#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#define _USE_MATH_DEFINES
#include <math.h>

double Pref=0.101325;  /* MPa */
double Tref=25.+273.15;/* K */
double R   =8.3144621; /* kJ/kg-mole/K */

typedef struct{double Hf,Sf,Cp;}HSC; /* kJ/kg/K */
HSC O2={     0.,205.2,28.01};
HSC O ={249336.,161.1,21.93};

double G(HSC hsc,double T,double P)
  {
  return((hsc.Hf+hsc.Cp*(T-Tref)-T*(hsc.Sf+hsc.Cp*log(T/Tref)-R*log(P/Pref)))/R/Tref);
  }

double Gibbs(double T,double P,double delta)
  {
  return((1.-delta)*G(O2,T,P*(1.-delta)/(1.+delta))
    +2.*delta*G(O,T,P*2.*delta/(1.+delta)));
  }

double dGd(double T,double P,double delta)
  {
  double d1,d2,g1,g2;
  if(delta<0.5)
    {
    d1=delta;
    d2=delta+0.00001;
    }
  else
    {
    d1=delta-0.00001;
    d2=delta;
    }
  g1=Gibbs(T,P,d1);
  g2=Gibbs(T,P,d2);
  return((g2-g1)/(d2-d1));
  }

double dmin(double T,double P)
  {
  int iter;
  double d1,d2,delta;
  d1=0.;
  d2=1.;
  for(iter=0;iter<64;iter++)
    {
    delta=(d1+d2)/2.;
    if(dGd(T,P,delta)<0.)
      d1=delta;
    else
      d2=delta;
    }
  return(delta);
  }

int main(int argc,char**argv,char**envp)
  {
  double delta,g,T;
  printf("T,delta,g\n");
  for(T=500.;T<6001.;T+=250.)
    {
    delta=dmin(T,Pref);
    g=Gibbs(T,Pref,delta);
    printf("%.0lf,%lf,%.1lf\n",T,delta,g);
    }
  return(0);
  }
